from fastapi import APIRouter

base_router = APIRouter()


@base_router.get("/", tags=["root"])
async def read_root() -> dict:
    return {"msg": "Welcome!", "status": "OK"}


@base_router.get("/health", tags=["health"])
async def health() -> dict:
    return {"msg": "API is working correctly", "status": "OK"}



import os


class settings:
    DEVELOP = bool("true" == str(os.getenv("DEVELOP", "false")).lower())
    LOG_LEVEL = os.getenv("LOG_LEVEL", "DEBUG")
    JSON_FORMAT = bool("true" == str(os.getenv("JSON_FORMAT", "false")).lower())
    EXCHANGE_{{ cookiecutter.project_name.upper() }} = os.getenv("EXCHANGE_{{ cookiecutter.project_name.upper() }}")
    EXCHANGE_TYPE_{{ cookiecutter.project_name.upper() }} = os.getenv("EXCHANGE_TYPE_{{ cookiecutter.project_name.upper() }}")
    QUEUE_{{ cookiecutter.project_name.upper() }} = os.getenv("QUEUE_{{ cookiecutter.project_name.upper() }}")
    POSTGRES_USER = os.getenv("POSTGRES_USER")
    POSTGRES_PASSWORD = os.getenv("POSTGRES_PASSWORD")
    POSTGRES_HOST = os.getenv("POSTGRES_HOST")
    POSTGRES_PORT = os.getenv("POSTGRES_PORT")
    POSTGRES_DB = os.getenv("POSTGRES_DB")



import contextlib
import logging
import os
import enum
import uuid
from typing import Any, AsyncIterator

from sqlalchemy.ext.asyncio import (
    AsyncConnection,
    AsyncSession,
    async_sessionmaker,
    create_async_engine,
)
from sqlalchemy import inspect, Identity, BigInteger
from sqlalchemy.orm import (
    DeclarativeBase,
    MappedAsDataclass,
    Mapped,
    mapped_column,
)


class Base(MappedAsDataclass, DeclarativeBase):
    id: Mapped[int] = mapped_column(
        BigInteger, Identity(always=True), init=False, primary_key=True
    )
    """subclasses will be converted to dataclasses"""

    def to_dict(self, recurse: bool = True, force_exclude: list = []) -> dict:
        """Convert an SQLAlchemy model instance to a dictionary."""
        if not hasattr(self, "__dict__"):
            return {}
        # Convert object attributes to dictionary
        result = {}
        for key, value in self.__dict__.items():
            if key != "_sa_instance_state" and key not in force_exclude:
                if isinstance(value, enum.Enum):
                    result[key] = value.value
                elif isinstance(value, uuid.UUID):
                    result[key] = str(value)
                else:
                    result[key] = value
        # Get relationships from the SQLAlchemy inspection system
        mapper = inspect(type(self))
        for relationship in mapper.relationships:
            if relationship.key not in result:
                continue
            value = getattr(self, relationship.key)

            if value is None:
                result[relationship.key] = None
            elif isinstance(value, list):
                if recurse:
                    result[relationship.key] = [
                        item.to_dict(recurse=recurse) for item in value
                    ]
                else:
                    result[relationship.key] = [item.id for item in value]
            else:
                if recurse:
                    result[relationship.key] = value.to_dict(recurse=recurse)
                else:
                    result[relationship.key] = value.id

        return result


logger = logging.getLogger(__name__)

DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
    os.getenv("POSTGRES_USER"),
    os.getenv("POSTGRES_PASSWORD"),
    os.getenv("POSTGRES_HOST"),
    os.getenv("POSTGRES_PORT"),
    os.getenv("POSTGRES_DB"),
)
logger.info(f"Connecting with conn string {DB_URL}")


class DatabaseSessionManager:
    def __init__(self, host: str, engine_kwargs: dict[str, Any] = {}):
        self._engine = create_async_engine(host, **engine_kwargs)
        self._sessionmaker = async_sessionmaker(
            autocommit=False, expire_on_commit=False, bind=self._engine
        )

    async def close(self):
        if self._engine is None:
            raise Exception("DatabaseSessionManager is not initialized")
        await self._engine.dispose()

        self._engine = None
        self._sessionmaker = None

    @contextlib.asynccontextmanager
    async def connect(self) -> AsyncIterator[AsyncConnection]:
        if self._engine is None:
            raise Exception("DatabaseSessionManager is not initialized")

        async with self._engine.begin() as connection:
            try:
                yield connection
            except Exception:
                await connection.rollback()
                raise

    @contextlib.asynccontextmanager
    async def session(self) -> AsyncIterator[AsyncSession]:
        if self._sessionmaker is None:
            raise Exception("DatabaseSessionManager is not initialized")

        session = self._sessionmaker()
        try:
            yield session
        except Exception:
            await session.rollback()
            raise
        finally:
            await session.close()


sessionmanager = DatabaseSessionManager(DB_URL)


async def get_db_session():
    async with sessionmanager.session() as session:
        yield session



import os
from cookiecutter.main import cookiecutter


def start():
    available_templates = ["api-scelet", "service-scelet"]
    for i, template in enumerate(available_templates):
        print(f"ID: {i} -- Name: {template}")
    while True:
        try:
            choice = int(input("Pick a template ID from the list: "))
        except ValueError:
            print("Value is not an int. Retrying..")
            continue
        if choice > len(available_templates) - 1:
            print("Wrong id.")
        else:
            break

    cookiecutter(
        template=available_templates[choice],
        directory=os.path.join(
            os.path.dirname(os.path.realpath(__file__)), available_templates[choice]
        ),
    )


if __name__ == "__main__":
    start()



def test_health(client):
    response = client.get("/api/health")
    print(response.status_code)
    print(response.json())
    assert response.status_code == 200
    assert response.json()["status"] == "OK"


def test_root(client):
    response = client.get("/api/")
    print(response.status_code)
    print(response.json())
    assert response.status_code == 200
    assert response.json()["status"] == "OK"



from typing import Annotated

from package.app.models import get_db_session
from fastapi import Depends
from sqlalchemy.ext.asyncio import AsyncSession

DBSessionDep = Annotated[AsyncSession, Depends(get_db_session)]



from mrkutil.responses import ServiceResponse
from mrkutil.base import BaseHandler
import logging

logger = logging.getLogger(__name__)


class ExampleHandler(BaseHandler):

    @staticmethod
    def name():
        return "test_method"

    def process(self, data, corr_id):
        self.data = data.get("request", None)
        logger.info("Handle example")
        return ServiceResponse(
            200, f"example response from service with request data={self.data}"
        )



import logging
import enum
import uuid

from package.app import settings
from sqlalchemy import create_engine, NullPool, inspect, Identity, BigInteger
from sqlalchemy.orm import (
    DeclarativeBase,
    sessionmaker,
    MappedAsDataclass,
    Mapped,
    mapped_column,
)


class Base(MappedAsDataclass, DeclarativeBase):
    id: Mapped[int] = mapped_column(
        BigInteger, Identity(always=True), init=False, primary_key=True
    )
    """subclasses will be converted to dataclasses"""

    def to_dict(self, recurse: bool = True, force_exclude: list = []) -> dict:
        """Convert an SQLAlchemy model instance to a dictionary."""
        if not hasattr(self, "__dict__"):
            return {}
        # Convert object attributes to dictionary
        result = {}
        for key, value in self.__dict__.items():
            if key != "_sa_instance_state" and key not in force_exclude:
                if isinstance(value, enum.Enum):
                    result[key] = value.value
                elif isinstance(value, uuid.UUID):
                    result[key] = str(value)
                else:
                    result[key] = value
        # Get relationships from the SQLAlchemy inspection system
        mapper = inspect(type(self))
        for relationship in mapper.relationships:
            if relationship.key not in result:
                continue
            value = getattr(self, relationship.key)

            if value is None:
                result[relationship.key] = None
            elif isinstance(value, list):
                if recurse:
                    result[relationship.key] = [
                        item.to_dict(recurse=recurse) for item in value
                    ]
                else:
                    result[relationship.key] = [item.id for item in value]
            else:
                if recurse:
                    result[relationship.key] = value.to_dict(recurse=recurse)
                else:
                    result[relationship.key] = value.id

        return result


logger = logging.getLogger(__name__)

DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
    settings.POSTGRES_USER,
    settings.POSTGRES_PASSWORD,
    settings.POSTGRES_HOST,
    settings.POSTGRES_PORT,
    settings.POSTGRES_DB,
)
logger.info(f"Connecting with conn string {DB_URL}")


engine = create_engine(DB_URL, pool_pre_ping=True, poolclass=NullPool)
Session = sessionmaker(autocommit=False, autoflush=True, bind=engine)


def get_db_session():
    try:
        db = Session()
        yield db
    finally:
        db.close()



import os
from contextlib import ExitStack

import pytest
import json
from alembic.config import Config
from alembic.migration import MigrationContext
from alembic.operations import Operations
from alembic.script import ScriptDirectory
from app.models import Base, get_db_session
from app.models.base import DatabaseSessionManager
from app.main import app as actual_app
from psycopg.connection_async import AsyncConnection as Connection

from fastapi.testclient import TestClient
from sqlalchemy.pool import NullPool
from sqlalchemy.ext.asyncio import create_async_engine, close_all_sessions
from sqlalchemy import text, inspect, create_engine

DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
    os.getenv("POSTGRES_USER"),
    os.getenv("POSTGRES_PASSWORD"),
    os.getenv("POSTGRES_HOST"),
    os.getenv("POSTGRES_PORT"),
    "test_db",
)


@pytest.fixture(autouse=True)
def app(event_loop):
    with ExitStack():
        yield actual_app


@pytest.fixture
def client(event_loop, app):
    with TestClient(app) as c:
        yield c


def run_migrations(connection: Connection):
    config = Config("app/alembic.ini")
    config.set_main_option("script_location", "app/alembic")
    config.set_main_option("sqlalchemy.url", DB_URL)
    script = ScriptDirectory.from_config(config)

    def upgrade(rev, context):
        return script._upgrade_revs("head", rev)

    context = MigrationContext.configure(
        connection, opts={"target_metadata": Base.metadata, "fn": upgrade}
    )

    with context.begin_transaction():
        with Operations.context(context):
            context.run_migrations()


@pytest.fixture(scope="session")
async def sessionmanager():
    yield DatabaseSessionManager(DB_URL, {"poolclass": NullPool})


@pytest.fixture(scope="session", autouse=True)
async def setup_database(sessionmanager):
    # Run alembic migrations on test DB
    ROOT_DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
        os.getenv("POSTGRES_USER"),
        os.getenv("POSTGRES_PASSWORD"),
        os.getenv("POSTGRES_HOST"),
        os.getenv("POSTGRES_PORT"),
        "postgres",
    )
    my_engine = create_async_engine(
        ROOT_DB_URL,
        json_serializer=lambda obj: json.dumps(obj, default=str),
        echo=True,
        isolation_level="AUTOCOMMIT",
        poolclass=NullPool,
    )  # connect to server
    async with my_engine.connect() as conn:
        await conn.execute(text("DROP DATABASE IF EXISTS test_db;"))  # create db
        await conn.execute(text("CREATE DATABASE test_db;"))  # create db
    async with sessionmanager.connect() as connection:
        await connection.run_sync(run_migrations)

    yield

    # Teardown
    await sessionmanager.close()


def truncate_all_tables(engine):
    inspector = inspect(engine)
    with engine.connect() as conn:
        conn.execute(text("SET session_replication_role = 'replica';"))
        for table_name in inspector.get_table_names():
            conn.execute(
                text(f'TRUNCATE TABLE "{table_name}" RESTART IDENTITY CASCADE;')
            )
        conn.execute(text("SET session_replication_role = 'origin';"))


@pytest.fixture(scope="function", autouse=True)
async def drop_tables():
    yield
    await close_all_sessions()
    TEST_DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
        os.getenv("POSTGRES_USER"),
        os.getenv("POSTGRES_PASSWORD"),
        os.getenv("POSTGRES_HOST"),
        os.getenv("POSTGRES_PORT"),
        "test_db",
    )
    my_engine = create_engine(
        TEST_DB_URL,
        echo=False,
        isolation_level="AUTOCOMMIT",
        poolclass=NullPool,
    )  # connect to server
    truncate_all_tables(my_engine)


# Each test function is a clean slate
@pytest.fixture(scope="function", autouse=True)
async def transactional_session(sessionmanager):
    async with sessionmanager.session() as session:
        try:
            session.autoflush = True
            yield session
        finally:
            await session.rollback()  # Rolls back the outer transaction


@pytest.fixture(scope="function")
async def db_session(transactional_session):
    yield transactional_session


@pytest.fixture(scope="function", autouse=True)
async def session_override(app, sessionmanager):
    async def get_db_session_override():
        async with sessionmanager.session() as session:
            try:
                session.autoflush = True
                yield session
            finally:
                await session.rollback()  # Rolls back the outer transaction

    app.dependency_overrides[get_db_session] = get_db_session_override



import os
import inspect
import logging
import uvicorn
from fastapi.middleware import Middleware
from contextlib import asynccontextmanager
from starlette.middleware.cors import CORSMiddleware
from package.app.models import (
    sessionmanager,
    get_db_session,
)
from package.app import api as a_app
from fastapi import FastAPI
from mrkutil.logging import get_logging_config

log_level = os.getenv("LOG_LEVEL", "DEBUG")
develop = bool("true" == str(os.getenv("DEVELOP", "false")).lower())
json_format = bool("true" == str(os.getenv("JSON_FORMAT", "false")).lower())

logging_config = get_logging_config(log_level, json_format, True)
logging.config.dictConfig(logging_config)
logger = logging.getLogger("main")

API_STR = os.getenv("API_ROOT", "/api")
develop = str(os.getenv("DEVELOP", "False")).lower() == "true"


@asynccontextmanager
async def lifespan(app: FastAPI):
    """
    Function that handles startup and shutdown events.
    To understand more, read https://fastapi.tiangolo.com/advanced/events/
    """
    yield
    if sessionmanager._engine is not None:
        # Close the DB connection
        await sessionmanager.close()


allowed_hosts = os.getenv("ALLOWED_HOSTS", "*")
if allowed_hosts != "*":
    allowed_hosts = allowed_hosts.split(",")


middleware = [
    Middleware(
        CORSMiddleware,
        allow_origins=allowed_hosts,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
        expose_headers=["*"],
    )
]

app = FastAPI(
    lifespan=lifespan,
    middleware=middleware,
    debug=develop,
)

for name, obj in inspect.getmembers(a_app):
    if "_router" in name:
        app.include_router(obj, prefix=API_STR)


def dev():
    uvicorn.run(
        "package.app.main:app",
        host="0.0.0.0",
        port=8080,
        reload=True,
        log_level=log_level.lower(),
        log_config=logging_config,
    )


def prod():
    uvicorn.run(
        "package.app.main:app",
        headers=[
            ("server", "Apache"),
            ("X-Frame-Options", "SAMEORIGIN"),
            ("X-XSS-Protection", "1; mode=block"),
            ("X-Content-Type-Options", "nosniff"),
            ("Strict-Transport-Security", "max-age=15768000; includeSubDomains"),
            ("Referrer-Policy", "no-referrer-when-downgrade"),
            ("Content-Security-Policy", "frame-ancestors 'self'"),
        ],
        host="0.0.0.0",
        port=80,
        log_level=log_level.lower(),
        log_config=logging_config,
        forwarded_allow_ips="*",
    )


class DBSessionContextManager:
    def __init__(self):
        self._session_generator = get_db_session()
        self._session = None

    async def __aenter__(self):
        self._session = await self._session_generator.__anext__()
        return self._session

    async def __aexit__(self, exc_type, exc, tb):
        await self._session.close()


if __name__ == "__main__":
    logger.info("Rest API application up and running!")
    dev() if develop else prod()



from mrkutil.communication import listen
from mrkutil.logging import get_logging_config
from package.app import handlers, settings  # noqa importing handlers

import os
import sys
import logging
import logging.config


logging.config.dictConfig(
    get_logging_config(settings.LOG_LEVEL, settings.JSON_FORMAT, False)
)

logger = logging.getLogger("main")


def main():
    """
    App starting point
    """
    try:
        pid = str(os.getpid())
        if not os.path.isdir("/tmp/service"):
            os.makedirs("/tmp/service")
        pidfile = "/tmp/service_{{ cookiecutter.project_name }}.pid"
        if os.path.isfile(pidfile):
            logger.warning("Service is already running")
            sys.exit(1)
        with open(pidfile, "w") as file:
            file.write(pid)
            file.write("\n")
        try:
            logger.info("Starting ...")
            listen(
                exchange=settings.EXCHANGE_{{ cookiecutter.project_name.upper() }},
                exchange_type=settings.EXCHANGE_TYPE_{{ cookiecutter.project_name.upper() }},
                queue=settings.QUEUE_{{ cookiecutter.project_name.upper() }},
            )
            sys.exit(0)
        finally:
            os.unlink(pidfile)
    except KeyboardInterrupt:
        logger.info("Reloading ...")


if __name__ == "__main__":
    if settings.DEVELOP:
        from watchfiles import run_process

        logger.info("Running watchfiles, will watch for changes in current service ...")
        run_process("./package", target=main)
    else:
        main()



import asyncio
from logging.config import fileConfig

from alembic import context
from package.app.models.base import Base, DB_URL
from psycopg.connection_async import AsyncConnection as Connection
from sqlalchemy import pool
from sqlalchemy.ext.asyncio import async_engine_from_config

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)  # type: ignore

# add your model's MetaData object here
target_metadata = Base.metadata


def include_object(object, name, type_, reflected, compare_to):
    if type_ == "table" and reflected and compare_to is None:
        return False
    else:
        return True


def run_migrations_offline():
    """Run migrations in 'offline' mode.
    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.
    Calls to context.execute() here emit the given string to the
    script output.
    """
    context.configure(
        url=DB_URL,
        target_metadata=target_metadata,
        version_table="user",
        include_object=include_object,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection: Connection) -> None:
    context.configure(
        connection=connection,
        target_metadata=target_metadata,
        include_object=include_object,
        version_table="{{cookiecutter.project_name}}",
    )

    with context.begin_transaction():
        context.run_migrations()


async def run_migrations_online():
    """Run migrations in 'online' mode.
    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    configuration = config.get_section(config.config_ini_section)
    configuration["sqlalchemy.url"] = DB_URL
    connectable = async_engine_from_config(
        configuration,
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migrations)

    await connectable.dispose()


if context.is_offline_mode():
    run_migrations_offline()
else:
    asyncio.run(run_migrations_online())



import pytest
import contextlib
from alembic.config import Config
from alembic.migration import MigrationContext
from alembic.operations import Operations
from alembic.script import ScriptDirectory

from sqlalchemy.pool import NullPool
from sqlalchemy import text, create_engine, inspect
from sqlalchemy.orm import close_all_sessions
from package.app import settings
from package.app.models import Base
from package.app.models.base import DB_URL


def truncate_all_tables(engine):
    inspector = inspect(engine)
    with engine.connect() as conn:
        conn.execute(text("SET session_replication_role = 'replica';"))
        for table_name in inspector.get_table_names():
            conn.execute(
                text(f'TRUNCATE TABLE "{table_name}" RESTART IDENTITY CASCADE;')
            )
        conn.execute(text("SET session_replication_role = 'origin';"))


def run_migrations(connection):
    config = Config("package/alembic.ini")
    config.set_main_option("script_location", "package/app/alembic")
    config.set_main_option("sqlalchemy.url", DB_URL)
    script = ScriptDirectory.from_config(config)

    def upgrade(rev, context):
        return script._upgrade_revs("head", rev)

    context = MigrationContext.configure(
        connection, opts={"target_metadata": Base.metadata, "fn": upgrade}
    )

    with context.begin_transaction():
        with Operations.context(context):
            context.run_migrations()


@pytest.fixture(scope="session", autouse=True)
def setup_database():
    # Run alembic migrations on test DB
    ROOT_DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
        settings.POSTGRES_USER,
        settings.POSTGRES_PASSWORD,
        settings.POSTGRES_HOST,
        settings.POSTGRES_PORT,
        "postgres",
    )
    my_engine = create_engine(
        ROOT_DB_URL,
        echo=False,
        isolation_level="AUTOCOMMIT",
        poolclass=NullPool,
    )  # connect to server
    with my_engine.connect() as conn:
        conn.execute(text("DROP DATABASE IF EXISTS test_db;"))
        conn.execute(text("CREATE DATABASE test_db;"))
    test_engine = create_engine(
        DB_URL,
        echo=False,
        isolation_level="AUTOCOMMIT",
        poolclass=NullPool,
    )
    with contextlib.closing(test_engine.connect()) as conn:
        run_migrations(connection=conn)


@pytest.fixture(scope="function", autouse=True)
def drop_tables():
    yield
    close_all_sessions()
    TEST_DB_URL = "postgresql+psycopg://{}:{}@{}:{}/{}".format(
        settings.POSTGRES_USER,
        settings.POSTGRES_PASSWORD,
        settings.POSTGRES_HOST,
        settings.POSTGRES_PORT,
        "test_db",
    )
    my_engine = create_engine(
        TEST_DB_URL,
        echo=False,
        isolation_level="AUTOCOMMIT",
        poolclass=NullPool,
    )  # connect to server
    truncate_all_tables(my_engine)
